function [ out,error ] = NXTInterface( in, varargin )
%NXTINTERFACE Handles the comunication with the NXT-brick.
%   NXTInterface is made to be used with RobotHandler, and is an interface
%   to a NXT-brick with associated NXC code. This interface will not work
%   with a NXT-brick without the NXC code, NXT SLAM, written for NXTInterface.
%   Use stings to call the desired function on the brick. Position and
%   orientation is stored as a rowvector: [x, y, theta] in [m] and in
%   [rad]. Drive length of robot is given in [cm]. An error message is
%   usualy given if any errors occur; if no error, an empty string is
%   returned as error message. e.g. to drive the robot forward 5 cm, use
%   the command [out, error] = NXTInterface('driveforward',5);

% Written by Trond K. Homestad april/may-13

error = '';
out = '';

persistent h;
persistent position;
persistent initializedNXTInterface;
persistent isConnected;

if(~ischar(initializedNXTInterface))   
    initializedNXTInterface = 'Initialized';
    position.x = 0;
    position.y = 0;
    position.t = 0;
end



% Working check for connection. Disabled for debugging.
% remove h from input arguemts
if(isempty(h)&&~strcmp(in,'connect')&&~strcmp(in,'disconnect')&&~strcmp(in,'setpath'))
    disp('Need to connect to NXT-brick')
    disp('first. Use "NXTInterface("connect")');
elseif(isempty(h)&&strcmp(in,'disconnect'))
    COM_CloseNXT all
    out = 'Disconnecting MatLab from NXT-brick.';
else

    switch lower(in)
        case {'test'}
            position.t
        case {'isbusy'}
            [out,error] = isBusy(h);
        case {'drivetopoint'}
            [out,error] = driveToPoint(varargin{:});
        case {'calibrateeopd'}
            [out, error] = calibration(h,varargin{:});
        case {'isconnected'}
            [out, error] = isRobotConnected(isConnected);
        case {'setpath'}
            setPath();
        case {'disconnect'}
            [isConnected,error] = disconnect(isConnected);
        case {'connect'}
            [h,isConnected,error] = connect(isConnected);
        case {'getrobotvariables','getvariables'}
            [out, error] = getRobotVariables();
        case {'driveforward'}
            [out, error] = driveForward(h,varargin{:}); % distance given in [cm]
            if(isempty(error))
                position = newPos(position,varargin{1}/100); % convert from [cm] to [m]
            end
        case {'rotaterobot'}
            ang = varargin{1};             
            [out,error] = rotateRobot(h,position.t,ang);
            if(isempty(error))
                position.t = ang*pi/180; % pos in rad. 
            end
        case {'rotatetower','turntower'}
            [out,error] = turnTower(varargin{:},h);
        case {'purgemailbox','purge'}
            [out,error] = purgeMailbox(h);
        case {'singlemeasurement','singlemes'}
            [out,error] = singleMes(h,varargin{:});
        case {'fullscan'}
            [out, error] = fullscan(h);
        case {'mesonscreen'}
            [out,error] = mesOnScreen(h);
        case {'manualdrive'}
            [out,error] = manualDrive(h,varargin{:});
        case {'ping','pingrobot'}
            [out,error] = pingRobot(h);
        case {'settowerangle','settower'}
            [out,error] = setTowerAngle(varargin{:},h);
        case {'getpos','getrobotpos'}
            % [out, error] = getPos(h);
            out = [position.x, position.y, position.t];
        case {'gettowerangle','gettower'}
            [out,error] = getTowerAngle(h,varargin{:});
        case {'setpos','setrobotpos'}
            if(length(varargin)==1)
                pos = varargin{1};
                %error = setRobotPos(x,y,h);
                position.x = pos(1);
                position.y = pos(2);
                position.t = pos(3);
            else
                error = 'Wrong arguments';
            end
        case {'setorientation'}
            theta = varargin{1};
            %error = setOrientation(theta,h);
            position.t = theta;
        case {'resetnxt','nxtreset','nxtresets'}
            %error = resetNXTValues(varargin{1},h);
            position.x = 0; 
            position.y = 0;
            position.t = 0;
        otherwise
            error = 'Wrong arguments, unknown function';
            out = 'Unknown function.'; 

    end
end %check for conenction


end


function [out,error] = isBusy(h)

out = '';
error = '';

msg = '53';
NXT_MessageWrite(msg,1,h);
pause(0.1);
rep = NXT_MessageRead(0,0,true);
if(strcmp(rep,'1'))
    % NXT is busy
    out = '1';
elseif(strcmp(rep,'0'))
    % not busy
    out = '';
else
    out = 'Error';
    error = 'Did not return proper data';
end


end

function [out, error] = driveToPoint(varargin)

out = '';
error = '';

if(nargin==2)
    angleIn = varargin{1};
    dist = varargin{2};

    [out1,error1] = NXTInterface('rotaterobot',angleIn);
    pause(0.2);
   
    while(~isempty(NXTInterface('isbusy'))) % empty means not busy
        pause(1);
    end
    [out2,error2] = NXTInterface('driveforward',dist);

    out = strcat('rotate: ',out1,' driveforward: ',out2);

    if(~isempty(strcat(error1,error2)))
        % error 
        error = strcat(error1,error2);
    end
else
    error = 'Wrong arguments';
end
end


function [out, error] = calibration(handle,varargin)
% Calibration of sensors, based on the RWTH file "CalibrateEOPD". Only
% "long range" mode is accepted and is set as default. Run calibration with two
% values.
out = '';
error = '';

if(isa(varargin{1},'float')&&(strcmp(varargin{2},'sensor1')||strcmp(varargin{2},'sensor2')) ...
        &&ischar(varargin{3}))
    distance = varargin{1};
    sensor = varargin{2}; % "sensor1" or "sensor2"
    calDist = varargin{3}; % "far" or "near"
    
    if(strcmp(sensor,'sensor1'))
        port = SENSOR_1; % sensor 1
    elseif(strcmp(sensor,'sensor2'))
        port = SENSOR_2; % sensor 2
    else
        error = 'error, input';
    end
    if(strcmp(calDist,'far'))
        calDist = 'FAR';
    elseif(strcmp(calDist,'near'))
        calDist = 'NEAR';
    else
        error = 'error, input';
    end
    if(isempty(error))
        CalibrateEOPD(port,calDist,distance);
    else
        out = 'Error, input';
        error = 'Invalid arguments. Use distance first, then sensor as a string';
    end
end



end

function [out, error] = rawToCmMeter(rawValIn,sensorNumber)
% Transform raw value of sensors to meters. Calibration is neccesary first.
% Based on files from RWTHMindstormsNXT-toolbox

% Less raw value in by distance. Inverse proportional to distance.

if(ischar(rawValIn))
    rawValIn = str2num(rawValIn);
end

% Obtain EOPD calibration

% Not finished: Check if value is in meters
out = '';
error = '';


error = '';

% Port is in range [0-3] and sensorNumber is [1-4], hence subtracting one.
calibMatrix = CalibrateEOPD(sensorNumber-1,'READMATRIX');

if nnz(isnan(calibMatrix)) == 0 && isfloat(calibMatrix) && nnz(size(calibMatrix) == [2 2]) == 2 && nnz(calibMatrix > 0) == 4

    % now follow http://www.hitechnic.com/blog/eopd-sensor/eopd-how-to-measure-distance/#more-178
    nearDist = calibMatrix(1, 1);
    nearEOPD = calibMatrix(1, 2);
    farDist  = calibMatrix(2, 1);
    farEOPD  = calibMatrix(2, 2);

    kScale = (farDist - nearDist) / (1 / sqrt(farEOPD) - 1 / sqrt(nearEOPD));
    
    kError = kScale / sqrt(nearEOPD) - nearDist; % they call it error, it's basically an offset
    
    
    
    rawVal = rawValIn;
    
    if rawVal == 0
        calcedDist = Inf;
    end
    calcedDist = kScale / sqrt(rawVal) - kError;
    
    if calcedDist < 0
        calcedDist = 0;
    end%if
    
    out = calcedDist;
    
    
end%if


end

function [out, error]= isRobotConnected(isConnected)
% Returns true or false if the robot is connected or not.

error = '';

if(isempty(isConnected))
    % Not connected
    out = false;
else
    out = true;
end

    

end

function [out,isConnected,error] = connect(isConnected)
% Connect to the NXT-robot.
error = '';
out = '';
    if(isConnected)
        error = 'Allready connected';
    else
        h = COM_OpenNXT('bluetooth.ini');
        COM_SetDefaultNXT(h);
        out = h;
        isConnected=true;
        calMat1 = [5 404;10 88];
        calMat2 = [5 404;10 88];
        CalibrateEOPD(0,'SETMATRIX',calMat1); % sensor1
        CalibrateEOPD(1,'SETMATRIX',calMat2); % sensor2
    end

end


function [out,error] = disconnect(isConnected)
% Disconnect from the NXT-robot.
out = '';
error = '';
    if(isConnected)
        COM_CloseNXT all
        clear all;
        error = '';
        isConnected = false;
        out = isConnected;
    else
        error = 'Allready disconnected';
    end
end


function position = newPos(position,lengthInMeter)
% Update the position of the robot. Input length is given in meters. 

% Current position and orientation.
theta = position.t;
pos = [position.x;position.y];

% Update position based on the forward motion. Position is updated from
% local frame to global frame. Theta is the angle beteween x-axes and is
% positive from global x-aksis to local x-axis.
rotmat=[cos(-theta) sin(-theta); -sin(-theta) cos(-theta) ];
pos = pos + rotmat*[lengthInMeter;0];

position.x = pos(1);
position.y = pos(2);

end


function [out , error ] = setPath()
% Include subfolders for the MatLab path.
out = '';
error = '';
    path(path,genpath(pwd));
    error = '';
end

function [out, error ] = resetNXTValues( alternative, handle )
%RESETNXTVALUES Resets either position, tower angle or all.
%   With arguments "NXTpos","NXTall" and "NXTtower", reset one of the
%   alternatives.

out = '';
error = '';

if( ischar(alternative))
    switch lower(alternative)
        case {'nxtall'}
            % reset all values on the nxt-brick
            msg = '502';
        case {'nxttower'}
            msg = '501';
        case {'nxtpos'}
            msg = '500';
        otherwise
            error = 'Invalid arguments';
    end
else
    error = 'Wrong arguments';
end

if(isempty(error))
    NXT_MessageWrite(msg,1,handle);
end




end




function [ out, error ] = getTowerAngle( handle, varargin )
% Returns the angle of the tower on the robot. Sensor 1 represents 0deg
% when its pointing straight forward.

error = '';
out = '';
if(~isempty(varargin))
    error = 'Wrong arguments';
else


    msg = '511';
    NXT_MessageWrite(msg,1,handle);

    tic;
    received = false;
    while(~received)
        pause(0.2);
        rep = NXT_MessageRead(0,0,true);
        if(rep)
            received = true;
        elseif(toc>8)
            error = 'Timeout';
        end
        out = rep;
    end
end



end

function [ out, error] = getRobotVariables()
% Return the parameters related to the robot.

% Parameters are updated at may-13.
out = '';
error = '';

LeftWheel = 0.056;
RightWheel = 0.056;
EGC = 0.05;
WheelBase = 0.136;

out = [ LeftWheel, RightWheel, WheelBase, EGC];
error = '';


end

function [ out, error ] = getPos( handle)
%GETPOS Gets the position from the NXT-robot
%   Requests and receive the position of the NXT-robot.

out = '';
error = '';

msg = '510';
NXT_MessageWrite(msg,1,handle);

tic;
received = false;
while(~received)
    pause(0.2);
    rep = NXT_MessageRead(0,0,true);
    if(rep)
        received = true;
    elseif(toc>8)
        error = 'Timeout';
    end
    out = rep;
end


end

function [ out, error ] = setTowerAngle( handle,varargin )
%SETTOWERANGLE Set the angle of the tower.
%       Set the internal value of hte tower on the NXT-brick. 
out = '';
error = '';

if(length(varargin)==1)
    angle = varargin{1};          
    error ='';
    if(angle<0||angle>360)
        error = 'Wrong arguments. Angle must be [0,360]';
    else
        msg = '521';
        msg = strcat(msg,num2str(angle));
        NXT_MessageWrite(msg,1,handle);
    end
else
    error = 'Wrong arguments for setTowerAngle function';
end



end

function [out, error ] = setOrientation( angle,handle )
%SETTOWERANGLE Set the orientation of the robot.
%       Set the internal value of the robots orientation on the NXT-brick. 
out = '';
error = '';
if(angle<0||angle>360)
    error = 'Wrong arguments. Angle must be [0,360]';
else
    msg = '522';
    msg = strcat(msg,num2str(angle));
    NXT_MessageWrite(msg,1,handle);
end

end


function [out, error ] = setRobotPos( x, y, handle )
%SETTOWERANGLE Set the internal position of the robot. 
%       Set the internal position (x,y).
out = '';
error = '';
if(isnumeric(x)&&isnumeric(y))
    msg = '520';
    msg = strcat(msg,num2str(x),'%',num2str(y));
    NXT_MessageWrite(msg,1,handle);
    %disp(msg);
else
    error = 'Wrong input arguments';
end

end







function [ out,error ] = pingRobot( handle, varargin )
%PINGROBOT Pings the robot.
 
error = '';
out = '';
if(~isempty(varargin))
    error = 'Wrong arguments for ping function.';
else
    msg = '4';
    NXT_MessageWrite(msg,1,handle);

    answered = false;
    tic;
    while(~answered)
        pause(0.2);
        rep = NXT_MessageRead(0,0,true);
        if(rep)
            answered = true;
            out = sprintf('Ping: %d',toc);
        elseif(toc>8)
            out = 'Timeout';
            answered = true;
        end
    end
end


end




function [out,error] =  mesOnScreen( handle )
%MESONSCREEN Show the measurement on-screen only.
%   Take a measurement with both sensors, and show the values on screen.
%   This value is raw value from the sensor.

out = '';
error = '';

msg = '13';
NXT_MessageWrite(msg,1,handle);


end

function [out,error ] = turnTower(handle,varargin)
%TURNTOWER Turn tower to given angle.
%   The NXT-robot setts the tower to the requested angle. 

out = '';
error = '';

if(length(varargin)==1)
    angle = varargin{1};
    if(angle<0||angle>360)
        error = 'Angle must be [0,360]';
    else
        msg = sprintf('02%0.3d30',angle);
        NXT_MessageWrite(msg,1,handle);
        error = '';
    end
else
    error = 'Wrong arguments';
end

end




function [out, error ] = manualDrive( handle,varargin )
%MANUALDRIVE Drive the robot manualy by specifying the direction.
%   Accepted directions are forward, w; stop, s; left, a; right, d; and
%   backwards, x.

out = '';
error = '';

if(length(varargin)==1)
    error = '';
    msg = '03';
    disp('Manual drive: ');
    switch lower(direction)
        case {'w'}
            % Forward
            disp('Forward');
            msg = strcat(msg,'w');
            %msg = strcat(msg,'119');
        case {'s'}
            % Stop
            %msg = strcat(msg,'s');
            msg = strcat(msg,'115');
        case {'a'}
            % Left
            msg = strcat(msg,'a');
        case {'d'}
            % Right
            msg = strcat(msg,'d');
        case {'x'}
            % Backwards
            msg = strcat(msg,'x');
        case {'l'}
            % Test for driving forward, tacho= 99999
            msg = strcat(msg,'l');
        case {'k'}
            % test for driving forward, tacho=-1
        otherwise
            error = 'Not a valid direction. Use "a", "d", "w", "s", ("l","k") and "x"';
    end
    NXT_MessageWrite(msg,1,handle);
else
    error = 'Wrong arguments';
end
out = '';


end




function [ out, error] = fullscan( handle)
%FULLSCAN Scans the suroundings of the robot. 
%   Scan the area of the robot. The positive x-axis of the robot is in
%   front so that the positive y-axis is to the left. This data, relative
%   to the robot, without considering the position and orientation, is
%   referred to as "local data". 
    

    % The following code requests data from the NXT-robot and replyes.
    % Depicted in the masterthesis Homestad 2013.
    purgeMailbox(handle);

    data1 = [];
    data2 = [];
    msg = '10'; % Request fullscan
    NXT_MessageWrite(msg,1,handle);
    tic;
    fullscanDone = false;
    timeout=false;
    pause(0.1);
   
    i_nxt = 0; % Package given from nxt
    i_MatLab = 0; % Package requested from MatLab
    measurement1 = -1;
    measurement2 = -1;
    error='';
    replyed = true; % true to send the first request to NXT
    out = [];
    error ='';
   
    
    while(~fullscanDone&&~timeout&&isempty(error))
        
        
        measurement1Str = '';
        measurement2Str = '';
        packetNumStr = '';
        
        
        % Request transfer number i. first is 0
        if(replyed)
            msg = sprintf('%i',i_MatLab);
            NXT_MessageWrite(msg,1,handle);
            replyed = false;
        end
        
        pause(0.1);
        rep = NXT_MessageRead(0,0,true);
        %sprintf(strcat('Reply from NXT: ',rep))
        
        if(~isempty(rep))
            replyed = true;
            if(strcmp(rep(1:3),'end'))
                fullscanDone = true;
                if(rep(4)=='0')
                    error = '';
                else
                    error = 'Error: error from NXT';
                end
            elseif(toc>35)
                timeout = true;
            elseif(rep(1)=='s')
                % parsing
                sens1Bool = false;
                sens2Bool = false;
                sensBool = false;
                packetNumBool  = false;
                error = '';
                for i_pars=1:length(rep)
                    ch = rep(i_pars);
                    if(error)
                        break;
                    elseif(ch=='s') % next number is sensor number
                        sensBool= true;
                    elseif(ch=='m') % end of measurement
                        sens1Bool=false;
                        sens2Bool=false;
                    elseif(ch=='f') % beginning of packet number
                        packetNumBool=true;
                    elseif(sensBool)
                        if(ch=='1')
                            sens1Bool = true; % next measurement from s1
                            sensBool = false;
                        elseif(ch=='2')
                            sens2Bool = true; % next measurement from s2
                            sensBool = false;
                        else
                            error='Error: parsing, sensor';
                        end
                    elseif(sens1Bool)
                        measurement1Str=strcat(measurement1Str,ch);
                    elseif(sens2Bool)
                        measurement2Str=strcat(measurement2Str,ch);
                    elseif(packetNumBool)
                        packetNumStr = strcat(packetNumStr,ch);
                    else
                        error = 'Error: parsing';
                    end

                end % end for
                
            if(~isempty(measurement1Str)&&~isempty(measurement2Str))    
                measurement1 = str2num(measurement1Str);
                measurement2 = str2num(measurement2Str);
            else
                measurement1 = -1;
                measurement2 = -1;
            end
            i_nxt = str2num(packetNumStr);    
            
            % end parsing
            %purgeMailbox(handle); % TEST
              
            if(i_nxt~=i_MatLab)
                error='Error: Wrong package'; % Wrong package.
            else
                data1(length(data1)+1,1) = measurement1;
                data2(length(data2)+1,1) = measurement2;  
                i_MatLab = i_MatLab+1;
            end
            end
        else
            % No reply from NXT
            replyed = false;
            pause(0.5);
        end
    end %end while
    % all data is recived, or an error has occured.
    
    % data1 is the measurements of sensor 1 and represents the degrees 
    % [0,175]. data2 is then the sensor 2 and represents deg [180,355],
    % both with 5 deg increments.
    s1 = data1;
    s2 = data2;
    for i=1:length(data1)
        % Converting rawdata to calibrated data.
        m1 = rawToCmMeter(data1(i),1);
        s1(i) = m1;
        m2 = rawToCmMeter(data2(i),2);
        s2(i) = m2;
    end

    
    % This part of the code is filtering the data so that measurements
    % longer than 25 cm is filtered out. And convert angle and length of
    % measurement to cartesian coordinates. 
    counter1 = 1;
    counter2 = 1;
    X1 = [];
    X2 = [];
    Y1 = [];
    Y2 = [];
    for i=1:180/5
        if(s1(i)<25)
            [X1(counter1),Y1(counter1)] = pol2cart(((i-1)*5)*(pi/180),s1(i));
            counter1 = counter1+1;
        end
        if(s2(i)<25)
            [X2(counter2),Y2(counter2)] = pol2cart((i-1+180/5)*5*(pi/180),s2(i));
            counter2=counter2+1;
        end

    end


    pos1 = [X1,X2]';
    pos2 = [Y1,Y2]';
    tmp = [pos1,pos2];

    if(isempty(tmp))
        data = [];
    else
        data = ones(length(tmp),3);
        data(:,2:3) = tmp;
        % First column represents the sensornumber. The number does not
        % need to reflect the actual number of sensor, rather the order of
        % which the measurements was taken. So the sorted list just set up
        % sensor number 1 to the length of the sensor-vector.
        data(:,1) = (1:length(data)); 

        

        % Tower should be directly between wheels. Remember x-axis is
        % forward on the robot, and then positive y-axis is to the left.
        % Add offset:
        data(:,2) = data(:,2)+3; % x-coordinates
        data(:,3) = data(:,3)-1; % y-coordinates
        
        out = data/100; % convert to meters, [m]     
    end
       
        
end
    
  


function [ data,error ] = singleMes( handle, varargin)
%SINGLEMES Gives single measurement from given sensor.
%   Requests measurement from the given sensor. Chose sensor 1 or 2.

    % purge mailbox
    purgeMailbox(handle);

    data= '';
    error = '';

    if(length(varargin)==1)
        sensor = varargin{1};
        sensor = lower(sensor);
    else
        error = 'Wrong arguments';
    end
    if isempty(error)
        if(strcmp(sensor,'sensor1'))
             msg = '11';
            NXT_MessageWrite(msg,1,handle);
            pause(0.2);
            raw = NXT_MessageRead(0,0,true,handle); % true, remove message from NXT
            %disp(fprintf('Raw value: %d',raw));
            data = rawToCmMeter(raw,1);
           
        elseif(strcmp(sensor,'sensor2'))
            msg = '12';
            NXT_MessageWrite(msg,1,handle);
            pause(0.2);
            data = NXT_MessageRead(0,0,true,handle); % true, remove message from NXT
            data = rawToCmMeter(data,2);
        else
            error = 'Wrong arguments singleMes, NXTI';
        end
    end
    
end



function [out,error] = purgeMailbox( h )
% Remove messages from inpox on the NXT-brick that might be there.
    out = '';
    error = 0;
    count = 0;
    try
    rep = NXT_MessageRead(0,0,true);
    while(~isempty(rep)&&~error)
        rep = NXT_MessageRead(0,0,true,h);
        pause(0.1);
        if(count>500)
            error = 1;
        end
    end
    catch
        disp('Error in Purge mailbox');
    end

end


function [out, error ] = rotateRobot(handle,pos,varargin)
%ROTATEROBOT Rotates robot to given angle.
%   Calls the rotate function on the NXT robot to rotate the robot to the
%   given angle. Angle is relative to world map. i.e. robot store the lokal
%   angle and computes apropriate rotation direction and distance. 
    
    out = '';
    error = '';

    if(length(varargin)==1)
        angleIn = varargin{1};
        power = 40;
    elseif(length(varargin)==2)
        angleIn = varargin{1};
        power = varargin{2};
    else
        error = 'Invalig arguments';
    end
    
    % IR-robot uses half the angle for some reason. Use the same half angle
    % and multiply the actual angle by two. 
    
    
    
    pos = pos*180/pi;
    
    if(isempty(error))
        if(angleIn<=360&&angleIn>=0)
        
            x = angleIn-pos;

            if(x>=180)   
                dir = 'left';
                x = 360-x;
                
            elseif(x<-180)
                dir = 'right';
                x = x+360;
                
            elseif(angleIn<pos)
                dir = 'left';
                x = x*-1;
                
            elseif(angleIn>pos)
                dir = 'right';
                
            else
                dir = 'noturn';
                
            end

            out = dir;
            x = int16(x);
            
            
            % add 100 to power. If power >= 100 it indicates a right turn on
            % the robot.
            if(strcmp(dir,'right'))
               power = power +100;
            end

            if(~strcmp(dir,'noturn'))
                msg = sprintf('01%0.3d%0.3d',x,power);
                
                NXT_MessageWrite(msg,1,handle);
            end
        else
            error = 'Value of angle must be [0,360]';
        end

    end
    
     

    

end


function [out ,error ] = driveForward( handle, varargin )
%DRIVEFORWARD Drive the robot forward the given distance with given power.
%   Calls the NXT robot to execute drive function with distance and power
%   as arguments. Value of distance must be [0,999] and value of power must
%   be [-100,100].

    out = '';
    error = '';

    if(length(varargin)==2)
        distanceIn = varargin{1};
        power = varargin{2};
    elseif(length(varargin)==1)
        power = 50;
        distanceIn = varargin{1};
    

    if(distanceIn<1000&&distanceIn>0&&power<101&&power>-101)
        msg = sprintf('00%0.3d%0.3d',distanceIn,power);
        NXT_MessageWrite(msg,1,handle);
    else
        error = 'Distance must be less than 1000 and bigger than 0. Power';
        error = strcat(error,' must be between -100 and 100.');
            
    end
    else
        error = 'Wrong input to driveForward, NXTI';
    end
    
end
