function [ out,error ] = RobotHandler( in, varargin )
%ROBOTHANDLER Handles the connection between CAS and robot interfaces.
% RobotHandler contains a set of standardized functions and metohods to
% standardize the dataflow between robots and CAS. RobotHandler comunicate
% with robots trough individual robot interfaces. Persistent variables
% makes this file able to store data, as which robot is connected.

% ver 1.0
% Written by Trond K. Homestad, may-13, NTNU


persistent robotConnected isError;

out = '';
error = '';

if(isError)
    % Error in robot
    % Use to tell if an error has occured in the robot. Coordinate with CAS
    % to handle error. Not implemented in CAS at this date.
    out = 'Robot has reportet an error.';
    error = 'checkForError has replyed with error.';
else

    switch lower(in)
        case {'ping'}                           % basisfunction
            [out, error] = ping();
        case {'getpos'}                         % basisfunction
            [out, error] = getPos();
        case {'getorientation'}
            [out, error] = getOrientation();
        case {'setpos'}                         % basisfunction
            [out, error] = setPos(varargin{1});
        case {'setorientation'}
            [out, error] = setOrientation(varargin);
        case {'fullscan'}                       % basisfunction
            [out, error] = fullScan();
        case {'driveforward'}                   % basisfunction
            [out, error] = driveForward(varargin);
        case {'rotate'}                         % basisfunction
            [out, error] = rotateToAngle(varargin{1});
        case {'drivetopoint'}
            if(length(varargin)==2)
                [out, error] = driveToPoint(varargin{1},varargin{2});
            else
                error = 'Need two arguments. First angle, then distance.';
            end
        case {'getrobottype'}
            [out,error] = getRobotType();
        case {'getrobotparams','getrobotpars','getrobotvariables'}                 % basisfunction
            [out,error] = getRobotParams();
        case {'isconnectedtorobot'}
            [out,error] = isConnectedToRobot();
        case {'connect'}                        % basisfunction
            [out,error] = connect();
        case {'disconnect'}                     % basisfunction
            [out, error] = disconnect();
        case {'iserror'}                        % basisfunction when implemented in CAS
            [out, error] = checkForError();
            isError = out;
        otherwise
            error = 'Invalid argument';

    end
end

function [out, error] = ping()
    % Ping the robot.
    out = '';
    error = '';
    switch robotConnected
        case 'NXTRobot'
            [out, error] = NXTInterface('ping');
        otherwise
            error = 'No connection';
    end
end

function [out, error] = getPos()
    % Get the position and orientation of the robot.
% Position given to CAS as [mm] and [rad] from RH. Convertion from [m] to
% [mm] is done here.
    out ='';
    error = '';

    switch robotConnected
        case 'NXTRobot'
            [pos, error] = NXTInterface('getpos'); % Output in [m] and rad
        otherwise 
            error = 'No connection';
    end
    
    % Convert to [mm] from [m].
    out = [pos(1)*1000,pos(2)*1000,pos(3)];
    
    
end

function [out, error] = getOrientation()
    % Get the orientation of the robot.

    out ='';
    error = '';

    switch robotConnected
        case 'NXTRobot'
            [out_bar, error] = NXTInterface('getpos'); % output in [m] and rad
            out = out_bar(3);
        otherwise 
            error = 'No connection';
    end
    
end

function [out, error] = setPos(position)
    % Sets the position of the robot.
% position from CAS is given in [mm] and [rad]. Position in RH is given in
% [m] and [rad]. 
xPos = position(1)/1000; % convert to [m]
yPos = position(2)/1000; % convert to [m]
theta = position(3);
    out ='';
    error = '';

    switch robotConnected
        case 'NXTRobot'
            if(length(position)==1)
                pos = position{1};
                [out, error] = NXTInterface('setpos',[xPos,yPos,theta]);
                
                
            else
                error = 'Wrong arguments at setpos()';
            end
        otherwise 
            error = 'No connection';
    end
    
end

function [out, error] = setOrientation(orientation)
    % Sets the internal orientation of the robot.
    out ='';
    error = '';
    switch robotConnected
        case 'NXTRobot'
            x = NXTInterface('getpos');
            x(3) = orientation;
            NXTInterface('setpos',x);
        otherwise 
            error = 'No connection';
    end
    
end

function [out, error] = fullScan()
    % Perform a 360deg scan of the area.
% Value of out is a vektor, [n x 3]. CAS need sensor number, so the first
% column is numbered 1 to length of array. The measurements is connected so
% that each number is next to each other. 1 is closer to 2 than to 3 etc.
% Column 2 and 3 is x and y values of local data to the robot. Where the x
% axis is in front, so that positive y-axis is to the left of the robot.
% Values in meters, [m].
error = '';
out = '';
    switch robotConnected
        case 'NXTRobot'
            [out, error] = NXTInterface('fullscan');
            
            if(~isempty(error))
               out = '';
            end
        otherwise 
            error = 'No connection';
    end
    
    
    
end

function [out, error] = driveForward(argIn)
    % Drive the robot forward given length in cm.
    out ='';
    error = '';

    switch robotConnected
        case 'NXTRobot'
            if(length(argIn)==1)
                dist = argIn{1};
                [out, error] = NXTInterface('driveforward',dist);
            else
                error = 'Wrong argument, driveForward';
            end
        otherwise 
            error = 'No connection';
    end
    
end

function [out, error] = rotateToAngle(angIn)
    % Rotate the robot to the double of the given angle in degrees.
    
    % IR-robot had a system of rotating double the input angle, and so the
    % angle here is doubled before the robot interfaces. The desired angle
    % of the robot is the double angle input here, i.e. desiredangle =
    % argIn*2. The CAS system is built to send half the angle, so its
    % better to dubble it here. 
    
    out ='';
    error = '';
    
    angIn = angIn*2;

    switch robotConnected
        case 'NXTRobot'
            if(length(argIn)==1)
                [out, error] = NXTInterface('rotaterobot',angIn);
            else
                error = 'Wrong argument, Rotate robot.';
            end
            
        otherwise 
            error = 'No connection';
    end
    
end

function [out, error] = driveToPoint(varargin)
    % Rotate the robot to the given angle, then drive forward the given
    % distance in degrees and cm.
    
    % as in the rotate function, the angle of rotation must be dubbled from
    % CAS to robot interfaces. 
    
    out ='';
    error = '';
    
    switch robotConnected
        case 'NXTRobot'
            if(length(varargin)==2)
                angle = varargin{1}*2;
                dist = varargin{2};
                NXTInterface('drivetopoint',angle,dist);
            else
                error = 'Wrong arguments, drive to point';
            end
            
        otherwise 
            error = 'No connection';
    end
    
end

function [out, error] = getRobotType()
    % Get the type of robot as a string to tell the user which robot is
    % connected in CAS.
    out ='';
    error = '';
    switch robotConnected
        case 'NXTRobot'
            out = 'NXTRobot';
            
        otherwise 
            error = 'No connection';
    end
    
end

function [out, error] = getRobotParams()
    % Return the parameters related to the robot.
    
    % out = [LW, RW, WB, EGC]
    % LW/RW- radius for left/right wheel
    % WB - length between wheels
    % EGC - Error growth coefficient.
    
    out ='';
    error = '';
    
    switch robotConnected
        case 'NXTRobot'
            [out, error] = NXTInterface('getvariables');
            
        otherwise 
            error = 'No connection';
    end
    
end

function [out, error] = isConnectedToRobot()
    % Returns true if a robot is connected, and false otherwise.
    out ='';
    error = '';

    switch robotConnected
        case 'NXTRobot'
            [out_bar,error] = NXTInterface('ping');
            if(strcmp(out_bar,'Timeout'))
                out = false;
                error='Timeout for NXT-robot. Not connected.';
            else
                out = true;
            end
        otherwise 
            error = 'No connection';
    end
    
end

function [out, error] = connect()
    % Connects to a robot. 
    
    out ='';
    error = '';
    
    if(~isempty(robotConnected))
        % Robot is connected in RH. Check if it is connected in NXTI.
        if(NXTInterface('isconnected'))
            % Robot is allready connected.
            out = strcat('Allready connected to: ',robotConnected);
            error = 'Allready connected, could not connect.';
        else
            % Connect to robot.
            [out,error] = NXTInterface('connect');
        end
    else
        % Connect to a robot
        [out, error] = NXTInterface('connect');
        
        if(isempty(error))
            robotConnected = 'NXTRobot';
            disp('Connected to NXTRobot.');
        else
            if(ischar(error))
                if(strcmp(error,'Allready connected'))
                    out = 'Allready connected';
                    robotConnected = 'NXTRobot';
                end
            else
                out = 'Did not connect to a robot.';
            end
        end
    end
    
end

function [out, error] = disconnect()
    % Disconnects from the connected robot. 
    out ='';
    error = '';
    
    if(~ischar(robotConnected))
        % Robot is allready disconnected in RH, check with NXTI
        robotConnected = '';
        out = 'Robot is allready disconnected';
        if(NXTInterface('isconnected'))
            % Robot is connected, disconnect from NXTI
            NXTInterface('disconnect');
            out = 'Disconnected from NXTRobot';
        end
    end
    switch robotConnected
        case 'NXTRobot'
            [out,error] = NXTInterface('disconnect');
            robotConnected = '';
            if(ischar(error)&&~isempty(error))
                if(strcmp(error,'Allready disconnected'))
                    disp('Robot allready disconnected');
                    robotConnected = '';
                end
            else
                disp('Disconnected from NXTRobot.');
            end
        otherwise 
            error = 'No connection';
            robotConnected = '';
    end
    
end

function [out, error] = checkForError()
    % Returns true or false if there is an error in the robot or not.
    out ='';
    error = '';

    switch robotConnected
        case 'NXTRobot'
            % No functionality for this exist yet.
            
        otherwise 
            error = 'No connection';
    end
    
end


end




